@@ -1079,14 +1079,13 @@ def doebuild(myebuild, mydo, _unused=DeprecationWarning, settings=None, debug=0,
 			if not fetch(fetchme, mysettings, listonly=listonly,
 				fetchonly=fetchonly, allow_missing_digests=False,
 				digests=dist_digests):
-				spawn_nofetch(mydbapi, myebuild, settings=mysettings,
-					fd_pipes=fd_pipes)
-				if listonly:
-					# The convention for listonly mode is to report
-					# success in any case, even though fetch() may
-					# return unsuccessfully in order to trigger the
-					# nofetch phase.
-					return 0
+				# Since listonly mode is called by emerge --pretend in an
+				# asynchronous context, spawn_nofetch would trigger event loop
+				# recursion here, therefore delegate execution of pkg_nofetch
+				# to the caller (bug 657360).
+				if not listonly:
+					spawn_nofetch(mydbapi, myebuild, settings=mysettings,
+						fd_pipes=fd_pipes)
 				return 1
 
 		if need_distfiles:
@@ -2046,7 +2045,7 @@ def _postinst_bsdflags(mysettings):
 def _post_src_install_uid_fix(mysettings, out):
 	"""
 	Files in $D with user and group bits that match the "portage"
-	user and group are automatically mapped to PORTAGE_INST_UID and
+	user or group are automatically mapped to PORTAGE_INST_UID and
 	PORTAGE_INST_GID if necessary. The chown system call may clear
 	S_ISUID and S_ISGID bits, so those bits are restored if
 	necessary.
@@ -2192,11 +2191,8 @@ def _post_src_install_uid_fix(mysettings, out):
 					mystat.st_ino not in counted_inodes:
 					counted_inodes.add(mystat.st_ino)
 					size += mystat.st_size
-
-				# Only remap the UID/GID if both match the portage user,
-				# in order to avoid interference with ebuilds that install
-				# files with portage group permissions (see bug 600804).
-				if (mystat.st_uid, mystat.st_gid) != (portage_uid, portage_gid):
+				if mystat.st_uid != portage_uid and \
+					mystat.st_gid != portage_gid:
 					continue
 				myuid = -1
 				mygid = -1
